@@ -2,10 +2,11 @@ package YAOO;
 use strict; no strict 'refs';
 use warnings;
 use Carp qw/croak/; use Tie::IxHash;
+use feature qw/state/;
+use Blessed::Merge;
+our $VERSION = '0.04';
 
-our $VERSION = '0.03';
-
-our (%object, %TYPES);
+our (%TYPES, %object);
 
 sub make_keyword {
 	my ($called, $key, $cb) = @_;
@@ -14,6 +15,7 @@ sub make_keyword {
 
 sub import {
 	my ($package, @attributes) = @_;
+
 	my $called = caller();
 
 	for my $is (qw/ro rw/) {
@@ -41,14 +43,70 @@ sub import {
 		});
 	}
 
-	make_keyword($called, 'auto_build', sub { $object{auto_build} = 1; });
+	make_keyword($called, 'auto_build', sub { $object{$called}{auto_build} = 1; });
 
-	$object{has} = {};
+	make_keyword($called, 'extends', sub {
+		my (@args) = @_;
+		my $extend = caller();	
+  		for my $inherit (@args) {
+			push @{*{\*{"${extend}::ISA"}}{ARRAY}}, $inherit;
+			return unless $object{$inherit};
+			my $bm = Blessed::Merge->new(blessed => 0, same => 0);
+			$object{$extend} = $bm->merge($object{$extend}, $object{$inherit});
+			for my $name (keys %{$object{$extend}{has}}) {
+				make_keyword($extend, $name, sub {
+					my ($self, $value) = @_;
+					if ($value && (
+						$object{$extend}{has}{$name}->{is} eq 'rw'
+							|| [split '::', [caller(1)]->[3]]->[-1] =~ m/^new|build|set_defaults|auto_build$/
+					)) {
+						$value = $object{$extend}{has}{$name}->{coerce}($value, $name)
+							if ($object{$extend}{has}{$name}->{coerce});
+						$object{$extend}{has}{$name}->{required}($value, $name)
+							if ($object{$extend}{$name}->{required});
+						$value = $object{$extend}{has}{$name}->{isa}($value, $name);
+						$self->{$name} = $value;
+						$object{$extend}{has}{$name}->{trigger}($value, $name)
+							if ($object{$extend}{has}{$name}->{trigger});
+					}
+					$self->{$name};
+				});
+			}
+			for my $name (keys %{$object{$extend}{method}}) {
+				make_keyword($extend, $name, $object{$called}{method}{$name});
+			}
+		}
+	});
 
-	*{"${called}::has"} = sub {
+	make_keyword($called, 'require_has', sub {
+		my (@args) = @_;
+		push @{ $object{$called}{require_has}  }, @args;
+	});
+
+	make_keyword($called, 'require_sub', sub {
+		my (@args) = @_;
+		push @{ $object{$called}{require_sub}  }, @args;
+	});
+
+	make_keyword($called, 'require_method', sub {
+		my (@args) = @_;
+		push @{ $object{$called}{require_sub}  }, @args;
+	});
+
+	$object{$called}{has} = {};
+	$object{$called}{method} = {};
+
+	make_keyword($called, "method", sub {
+		my ($name, $sub) = @_;
+
+		$object{$called}{method}{$name} = $sub;
+		make_keyword($called, $name, $sub);
+	});
+
+	make_keyword($called, 'has', sub {
 		my ($name, @attrs) = @_;
 
-		if ( $object{has}{$name} ) {
+		if ( $object{$called}{has}{$name} ) {
 			croak sprintf "%s attribute already defined for %s object.", $name, $called;
 		}
 
@@ -56,63 +114,88 @@ sub import {
 			croak sprintf "Invalid attribute definition odd number of key/value pairs (%s) passed with %s in %s object", scalar @attrs, $name, $called;
 		}
 
-		$object{has}{$name} = {@attrs};
+		$object{$called}{has}{$name} = {@attrs};
 
-		$object{has}{$name}{is} = 'rw'
-			if (! $object{has}{$name}{is});
+		$object{$called}{has}{$name}{is} = 'rw'
+			if (! $object{$called}{has}{$name}{is});
 
-		$object{has}{$name}{isa} = $TYPES{all}
-			if (not defined $object{has}{$name}{isa});
+		$object{$called}{has}{$name}{isa} = $TYPES{all}
+			if (not defined $object{$called}{has}{$name}{isa});
 
-		if ($object{has}{$name}{default}) {
-			if ($object{has}{$name}{default} =~ m/^1$/) {
-				$object{has}{$name}{value} = $object{has}{$name}{build_default}();
-			} elsif (ref $object{has}{$name}{default} eq 'CODE') {
-				$object{has}{$name}{value} = $object{has}{$name}{default}();
+		if ($object{$called}{has}{$name}{default}) {
+			if ($object{$called}{has}{$name}{default} =~ m/^1$/) {
+				$object{$called}{has}{$name}{value} = $object{$called}{has}{$name}{build_default}();
+			} elsif (ref $object{$called}{has}{$name}{default} eq 'CODE') {
+				$object{$called}{has}{$name}{value} = $object{$called}{has}{$name}{default}();
 			} else {
-				$object{has}{$name}{value} = $object{has}{$name}{type} eq 'ordered_hash'
-					? deep_clone_ordered_hash($object{has}{$name}{default})
-					: deep_clone($object{has}{$name}{default});
+				$object{$called}{has}{$name}{value} = $object{$called}{has}{$name}{type} eq 'ordered_hash'
+					? deep_clone_ordered_hash($object{$called}{has}{$name}{default})
+					: deep_clone($object{$called}{has}{$name}{default});
 			}
 		}
 
-		*{"${called}::$name"} = sub {
+		make_keyword($called, $name, sub {
 			my ($self, $value) = @_;
 			if ($value && (
-				$object{has}{$name}->{is} eq 'rw'
+				$object{$called}{has}{$name}->{is} eq 'rw'
 					|| [split '::', [caller(1)]->[3]]->[-1] =~ m/^new|build|set_defaults|auto_build$/
 			)) {
-				$value = $object{has}{$name}->{coerce}($value, $name)
-					if ($object{has}{$name}->{coerce});
-				$object{has}{$name}->{required}($value, $name)
-					if ($object{$name}->{required});
-				$object{has}{$name}->{isa}($value, $name);
+				$value = $object{$called}{has}{$name}->{coerce}($value, $name)
+					if ($object{$called}{has}{$name}->{coerce});
+				$object{$called}{has}{$name}{required}($value, $name)
+					if ($object{$called}{$name}->{required});
+				$value = $object{$called}{has}{$name}{isa}($value, $name);
 				$self->{$name} = $value;
-				$object{has}{$name}->{trigger}($value, $name)
-					if ($object{has}{$name}->{trigger});
+				$object{$called}{has}{$name}{trigger}($value, $name)
+					if ($object{$called}{has}{$name}->{trigger});
 			}
 			$self->{$name};
-		};
-	};
+		});
+	});
 
-
-	*{"${called}::new"} = sub {
+	make_keyword($called, "new", sub {
 		my ($pkg) = shift;
 		my $self = bless { }, $pkg;
-		set_defaults($self);
-		auto_build($self, @_) if ($object{auto_build});
+		require_has($called);
+		require_sub($self, $called);
+		require_method($called);
+		set_defaults($self, $called);
+		auto_build($self, @_) if ($object{$called}{auto_build});
 		$self->build(@_) if ($self->can('build'));
 		return $self;
-	};
+	});
+}
 
+sub require_has {
+	my ($called) = shift;
+	for (@{ $object{$called}{require_has} }) {
+		croak sprintf "The required %s attribute is not defined in the %s object.", $_, $called
+			if (! $object{$called}{has}{$_} );
+	}
+}
+
+sub require_sub {
+	my ($self, $called) = @_;
+	for (@{ $object{$called}{require_sub} }) {
+		croak sprintf "The required %s sub is not defined in the %s object.", $_, $called
+			if (! $self->can($_) );
+	}
+}
+
+sub require_method {
+	my ($called) = shift;
+	for (@{ $object{$called}{require_method} }) {
+		croak sprintf "The required %s method is not defined in the %s object.", $_, $called
+			if (! $object{$called}{method}{$_} );
+	}
 }
 
 sub set_defaults {
-	my ($self) = @_;
-	(defined $object{has}{$_}{value} && $self->$_($object{has}{$_}{type} eq 'ordered_hash'
-		? deep_clone_ordered_hash($object{has}{$_}{value})
-		: deep_clone($object{has}{$_}{value})
-	)) for keys %{$object{has}};
+	my ($self, $called) = @_;
+	(defined $object{$called}{has}{$_}{value} && $self->$_($object{$called}{has}{$_}{type} eq 'ordered_hash'
+		? deep_clone_ordered_hash($object{$called}{has}{$_}{value})
+		: deep_clone($object{$called}{has}{$_}{value})
+	)) for keys %{$object{$called}{has}};
 	return $self;
 }
 
@@ -179,6 +262,18 @@ sub scalarref {
 
 sub build_boolean { \0 }
 
+sub boolean {
+	my ($value, $name) = @_;
+	if (! ref $value) {
+		$value = \!!$value;
+	}
+	if (ref $value ne 'SCALAR' ) {
+		croak sprintf "The value passed to the %s attribute does not match the scalarref constraint.",
+			$name;
+	}
+	return $value;
+}
+
 sub build_ordered_hash { { } }
 
 sub ordered_hash { hash(@_); }
@@ -252,7 +347,7 @@ YAOO - Yet Another Object Orientation
 
 =head1 VERSION
 
-Version 0.03
+Version 0.04
 
 =cut
 
@@ -276,12 +371,25 @@ Version 0.03
 		oistins => 3
 	));
 
+	1;
+
 	...
 
 	Synopsis->new( satelites => 5 );
 
 	$synopsis->mind->{oistins};
 
+
+	...
+
+	package Life;
+
+	extends 'Synopsis';
+
+	requires_has qw/moon stars satellites mind/
+
+	1;
+
 =cut
 
 =head1 keywords
@@ -368,6 +476,18 @@ Version 0.03
 
 =cut
 
+=head2 extends
+
+=cut
+
+=head2 requires_has
+
+=cut
+
+=head2 requires_sub
+
+=cut
+
 =head1 AUTHOR
 
 LNATION, C<< <email at lnation.org> >>
